import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.StringTokenizer;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class deals with all editor actions. Basic functions such as 
 * open, save or print are processed here.
 * 
 * @author Daniel Brandstetter
 *
 */
public class ActionPerform {
	
	public FindDialog fDialog = null;
	public AboutDialog aDialog = null;
	public RhymeTemplateDialog tDialog = null;
	
	int returnVal;
	int option;
	
	String fileName = null;
	String content = null;
	
	public GedichteEditor ge;
	public Dictionary dict;
	
	JFileChooser jfc = new JFileChooser(); 

	/**
	 * Constructor
	 * @param ge: access to the main frame  
	 * @param dict: dictionary class, used by rhyme template
	 */
	public ActionPerform(GedichteEditor ge, Dictionary dict) {
		
		this.ge = ge;
		this.dict = dict;
	}

	/**
	 * Generates a new file
	 */
	public void newFile() {
		
		if(!ge.editor.getText().equals("") && !ge.editor.getText().equals(content)) {
			
			option = JOptionPane.showConfirmDialog(null, ge.BUNDLE.getString("ActionPerform.save.confirmation"));
            if(option == 0) {
            	
            	//If name is null, show saveAs, else save dialog
            	if(fileName == null) saveAs();
            	else save();
                ge.editor.setText("");
             }
             if(option == 1) {
            	 ge.editor.setText("");
             }
             
		} else {
			ge.editor.setText("");
		}
		
		//Reset the title
		ge.frmGedichteeditor.setTitle(ge.BUNDLE.getString("GedichteEditor.new.title")+" - GedichteEditor");
		fileName = null;
	      
	}
	
	/**
	 * Opens text files
	 */
	public void openFile () {
		
		//If the text is not null and the text is not the same as before
		if(!ge.editor.getText().equals("") && !ge.editor.getText().equals(content)) {
			
			option = JOptionPane.showConfirmDialog(null, ge.BUNDLE.getString("ActionPerform.save.confirmation"));
	        if(option == 0) {
	        	
	        	//If name is null, show saveAs, else save dialog
				if(fileName == null) saveAs();
				else save();
	    		open();
	        }
	        if(option == 1) {
	        	open();
	        }

		} else {
			open();
		}
	}

	/**
	 * Distinction between save and save as
	 */
	public void saveFile() {
		
		if(fileName == null) saveAs(); 
		else save();
	}

	/**
	 * Save as
	 */
	public void saveFileAs() {
		saveAs();
	}

	/**
	 * Generates the print class
	 */
	public void print() {
		PrintClass pc = new PrintClass(ge.editor, ge);
		pc.print();
	}
	
	/**
	 * Cut order
	 */
	public void cutFile() {
		ge.editor.cut();
	}
	
	/**
	 * Copy order
	 */
	public void copyFile() {
		ge.editor.copy();
	}
	
	/**
	 * Paste order
	 */
	public void pasteFile() {
		ge.editor.paste();
	}
	
	/**
	 * Delete the selected text
	 */
	public void deleteSelectedText() {
		ge.editor.replaceSelection("");
	}
	
	/**
	 * Exit application, show save dialog if text is not saved.
	 */
	public void exitApplication() {
		
		if(!ge.editor.getText().equals("") && !ge.editor.getText().equals(content)) {
			
			option = JOptionPane.showConfirmDialog(null, ge.BUNDLE.getString("ActionPerform.save.confirmation"));
            if(option == 0) {
            	
            	//If name is null, show saveAs, else save dialog
            	if(fileName == null)saveAs();
            	else save();
            	System.exit(0);
            }
            if(option == 1) {
            	System.exit(0);
            }
            
		} else {
			System.exit(0);
		}
	}
	
	/**
	 * Select all order
	 */
	public void selectAll() {
		ge.editor.selectAll();
	}
	
	/**
	 * Find text, opens the dialog to search for words
	 */
	public void findText() {
		
		//Initialice only once
		if (fDialog  == null) {
			fDialog = new FindDialog(ge);
		}
		
		fDialog.setLocationRelativeTo(ge.frmGedichteeditor);
		ge.addEscapeListener(fDialog);
		fDialog.setVisible(true);
	}
	
	/**
	 * Opens the rhyme dialog
	 */
	public void rhymeTemplate() {
		
		//Initialice only once
		if (tDialog  == null) {
			tDialog = new RhymeTemplateDialog(this.ge, this.dict);
		}
		tDialog.setLocationRelativeTo(ge.frmGedichteeditor);
		ge.addEscapeListener(tDialog);
		tDialog.setVisible(true);	
	}
	
	/**
	 * Opens the about dialog
	 */
	public void about() {
		
		//Initialice only once
		if (aDialog  == null) {
			aDialog = new AboutDialog(this.ge);
		}
		aDialog.setLocationRelativeTo(ge.frmGedichteeditor);
		ge.addEscapeListener(aDialog);
		aDialog.setVisible(true);	
	}
	
	/**
	 * Opens the accentution dialog
	 */
	public void accentution(int index) {
		
		AccentuationDialog acDialog = new AccentuationDialog(this.ge, index);
		acDialog.setLocationRelativeTo(ge.frmGedichteeditor);
		ge.addEscapeListener(acDialog);
		acDialog.setVisible(true);	
		
	}
	
	/**
	 * Open Dialog, to open any text file
	 */
	public void open() {
		
		jfc.setDialogTitle(ge.BUNDLE.getString("ActionPerform.open.title"));
		returnVal = jfc.showOpenDialog(null);
		if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	  
			ge.editor.setText(null);
	    	  
			try {
				//File name and path
				fileName = jfc.getSelectedFile().getPath();
				
				FileInputStream f = new FileInputStream(jfc.getSelectedFile());
				FileChannel fc = f.getChannel();
				long fsize = fc.size();
				ByteBuffer buff = ByteBuffer.allocate((int)fsize);
				//Reads the bytes from channel to byte buffer
				fc.read(buff);
				buff.rewind();
				//Returns the byte buffer into an array and converts this array into string 
				String str = new String(buff.array());
				  
				//Append text to the pane
				ge.editor.setText(str);
				content = ge.editor.getText();
				
				//Close channel and stream
				fc.close();
				f.close();
				
			} catch(IOException ioe) {
				System.err.println("I/O Error on Open");
			}
			
			//Sets the title
			ge.frmGedichteeditor.setTitle(jfc.getSelectedFile().getName()+" - GedichteEditor");
		}
	}
	
	
	/**
	 * Save Dialog
	 */
	public void save() {
		
		jfc.setDialogTitle(ge.BUNDLE.getString("ActionPerform.save.title"));
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			
			try {
				
				FileOutputStream f = new FileOutputStream(jfc.getSelectedFile());
	            PrintWriter pw = new PrintWriter(f);
	            content = ge.editor.getText();
	            
	            StringTokenizer st = new StringTokenizer(content, System.getProperty("line.separator"));
	            while(st.hasMoreTokens()) {
            		pw.println(st.nextToken());
	            }
	           
	            //Close Stream and Writer
	            pw.close();
	            f.close();
	         
			} catch(IOException ioe) {
				System.err.println("I/O Error on Save");
			}
			
			//Sets the title
			ge.frmGedichteeditor.setTitle(jfc.getSelectedFile().getName()+" - GedichteEditor");
	      }
	}
	
	
	/**
	 * SaveAs Dialog
	 */
	public void saveAs() {
		
		jfc.setDialogTitle(ge.BUNDLE.getString("ActionPerform.saveas.title")); 
		returnVal = jfc.showSaveDialog(null);

		if(returnVal == JFileChooser.APPROVE_OPTION) {
			
			try {
				
				FileOutputStream f = new FileOutputStream(jfc.getSelectedFile() + ".txt");
	            PrintWriter pw = new PrintWriter(f);
	            content = ge.editor.getText();
	            fileName = jfc.getSelectedFile().getPath();
	            
	            StringTokenizer st=new StringTokenizer(content,System.getProperty("line.separator"));
	            while(st.hasMoreTokens()) {
            		pw.println(st.nextToken());
	            }

	            //Close Stream and Writer
	            pw.close();
	            f.close();
	            
			} catch(IOException ioe) {
				System.err.println ("I/O Error on Save");
			}
			
			//Sets the title
			ge.frmGedichteeditor.setTitle(jfc.getSelectedFile().getName()+" - GedichteEditor");
	      } 
	}
	
	/**
	 * Get selected text from area
	 */
	public String getSelectedText() {
		
		return ge.editor.getSelectedText();
	}
}
