import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

/**
 * Copyright 2013 Daniel Brandstetter
 * 
 * GedichteEditor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  GedichteEditor is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with GedichteEditor.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This class processes the metric. It analysis the specific text line
 * and returns the corresponding verse. The lines are broken down into 
 * their constituent syllables and visualized accordingly.
 * 
 * @author Daniel Brandstetter
 *
 */
public class Metric {
	
	public GedichteEditor ge;
	public Map<String, String> wordlist;
	Color green = new Color(212, 248, 198);
	Color rot = new Color(255, 212, 218);
	Color green_ = new Color(130, 246, 85);
	Color rot_ = new Color(248, 122, 139);

	/**
	 * Constructor
	 * @param ge: access to the main frame  
	 * @param wordlist: list with all dictionary entries
	 */
	public Metric(GedichteEditor ge, Map<String, String> wordlist) {
		this.ge = ge;
		this.wordlist = wordlist;
	}

	/**
	 * Sets the lines of the metric list, according to the number of 
	 * syllables
	 * @param text: text which is to be analyzed
	 */
	public void analyseText(String text) {
		
		String[] lines = null;
		lines = text.split("\n");
		
		setListMetric(lines);
		restStyleTextBackground();
		
	}
	
	/**
	 * Sets the syllables counter of each line
	 * @param lines: array of lines which will be analyzed
	 */
	@SuppressWarnings("unchecked")
	public void setListMetric(String[] lines) {
		
		int lineCount = lines.length;
		ge.listModelMetric.clear();
		String text = "";
		int count = 0;
		
		int length = ge.editor.getDocument().getLength();
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
    	
        if (!(text.equals("") || text.equals(" "))) {
			for (int i = 1; i <= lineCount; i++) {
				count = getHyphCount(i-1);
				if (count == 0) ge.listModelMetric.addElement(i+":");
				else ge.listModelMetric.addElement(i+": "+count+" "+ge.BUNDLE.getString("Default.syllables"));
			}
        }		
	}
	
	/**
	 * The text has been changed, and maybe also the syllables. 
	 * This method updates the syllables count.
	 * @param text:text which is to be analyzed
	 */
	public void updateText(String text) {
		
		String[] lines = null;
		lines = text.split("\n");
		
		updateListMetric(lines);
		restStyleTextBackground();
		
	}
	
	/**
	 * The text has been changed, and maybe also the syllables. 
	 * This method updates the list lines, only if they have
	 * changed. Existing lines remain.  
	 * @param lines: array of lines which will be analyzed
	 */
	@SuppressWarnings("unchecked")
	public void updateListMetric(String[] lines) {
		
		int lineCount = lines.length;
		String text = "";
		int count = 0;
		int loop_c = 0;
		List<Integer> remove = new ArrayList<Integer>();
		int size_lmm = ge.listModelMetric.size();
		
		int length = ge.editor.getDocument().getLength();
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        
        if (size_lmm >= lineCount) loop_c = size_lmm;
        else loop_c = lineCount;
        
        System.out.println(size_lmm);
        System.out.println(loop_c);
        
        if (!(text.equals("") || text.equals(" "))) {
			for (int i = 0; i < loop_c; i++) {
				
				if (i+1 <= lineCount) {
					if (i+1 <= size_lmm) {
						count = getHyphCount(i);
						
						if (ge.textList.get(i) != null && ge.textList.get(i).equals(this.getLine(i))) {
						} else {
							if (count == 0) ge.listModelMetric.set(i, i+1+":");
							else ge.listModelMetric.set(i, i+1+": "+count+" "+ge.BUNDLE.getString("Default.syllables"));
							ge.accentuationList.remove(i);
							ge.textList.remove(i);
						}
						
					} else {
						count = getHyphCount(i);
						if (count == 0) ge.listModelMetric.addElement(i+1+":");
						else ge.listModelMetric.addElement(i+1+": "+count+" "+ge.BUNDLE.getString("Default.syllables"));
					}
				} else if (i+1 <= size_lmm) {
					remove.add(i);
					ge.textList.remove(i);
					ge.accentuationList.remove(i);
				}
			}
			
			//Remove
			Collections.reverse(remove);
			for (int i = 0; i < remove.size(); i++) {
				ge.listModelMetric.remove(remove.get(i));
			}
			
			
			ge.listMetric.setEnabled(true);
        } else {
        	ge.listModelMetric.clear();
        }
		
	}
	
	/**
	 * Analysis the given stresses
	 * @param metric: Line with the corresponding stresses
	 * @return: the metric of the line
	 */
	public String getMetric(List<String> metric) {

		int check = 0;
		String m = "";
		
		//Alexandriner
		String[] alexandriner_12 = {"e", "-", "e", "-", "e", "-", "e", "-", "e", "-", "e", "-"};
		String[] alexandriner_13 = {"e", "-", "e", "-", "e", "-", "e", "-", "e", "-", "e", "-", "e"};
		
		//Blankvers
		String[] blankvers_10 = {"e", "-", "e", "-", "e", "-", "e", "-", "e", "-"};
		String[] blankvers_11 = {"e", "-", "e", "-", "e", "-", "e", "-", "e", "-", "e"};
		
		//Elfsilber
		//String[] elfsilber_11 = {"e", "-", "e", "-", "e", "-", "e", "-", "e", "-", "e"};
		
		//Hexameter
		String[] hexameter_17 = {"-", "e", "e", "-", "e", "e", "-", "e", "e", "-", "e", "e", "-", "e", "e", "-", "e"};
		//Spodeus???
		
		//Pentameter
		String[] pentameter_14 = {"-", "e", "e", "-", "e", "e", "-", "-", "e", "e", "-", "e", "e", "-"};
		
		
		switch (metric.size()) {
			case 10:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(blankvers_10[i])) {
						check += 1;
					}
				}
				if (check == 10) return ge.BUNDLE.getString("metric.blankverse");
				else return m;
			case 11:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(blankvers_11[i])) {
						check += 1;
					}
				}
				if (check == 11) return ge.BUNDLE.getString("metric.hendecasyllable");
				else return m;
			case 12:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(alexandriner_12[i])) {
						check += 1;
					}
				}
				if (check == 12) return ge.BUNDLE.getString("metric.alexandrian");
				else return m;
			case 13:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(alexandriner_13[i])) {
						check += 1;
					}
				}
				if (check == 13) return ge.BUNDLE.getString("metric.alexandrians");
				else return m;
			case 14:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(pentameter_14[i])) {
						check += 1;
					}
				}
				if (check == 14) return ge.BUNDLE.getString("metric.pentameter");
				else return m;
			case 17:
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
					if (metric.get(i).equals(hexameter_17[i])) {
						check += 1;
					}
				}
				if (check == 17) return ge.BUNDLE.getString("metric.stricthexameter");
				else return m;
			default:
				
				for (int i = 0; i < metric.size(); i++) {
					m = m + metric.get(i);
				}
				return m;
		}
	}

	/**
	 * Gets the syllables count of a specific line.
	 * @param index: index of the line
	 * @return
	 */
	public int getHyphCount(int index) {
		
		int length = ge.editor.getDocument().getLength();
		String text = "";
		int start = 0;
        int end = 0;
        int count = 0;
        int start_index = 0;
        int end_index = 0;
        String[] words = null;
        String line = "";
        String hyph = "";
        String[] hyphList = null;
        int hyphCount = 0;
		
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        
        if (text.indexOf("\n", start_index) == -1) {
        	start = 0;
        	end = text.length();
        }
        
        while (text.indexOf("\n", start_index) != -1) {
            
        	if (count == index) {
        		
        		start = start_index;
        		end_index = text.indexOf("\n", start_index+1);
        		if (end_index != -1) end = end_index;
        		else end = text.length();
        		break;
        	}
        	
        	start_index = text.indexOf("\n", start_index + 1);
        	count ++;
        }
        
        line = text.substring(start, end);
        words = line.split(" ");
        
        if (end-start > 1) {
	        for (int i = 0; i < words.length; i++) {
	
	            String word = words[i].replaceAll("[^a-zA-Z]", "");
	            if (wordlist.get(word) != null) {
	            	hyph = wordlist.get(word);
	            	hyphList = hyph.split("-");
	            } else {
	            	hyphList = word.split("  ");
	            }
	            
	            hyphCount += hyphList.length;
	        }
        }
        return hyphCount;
	}
	
	/**
	 * Gets the text line which is required
	 * @param index: index of the text line
	 * @return
	 */
	private String getLine(int index) {
		
		int start = 0;
        int end = 0;
        int count = 0;
        int start_index = 0;
        int end_index = 0;
        String text = "";
        String line = "";
        
        int length = ge.editor.getDocument().getLength();
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        

        //Only one line
        if (text.indexOf("\n", start_index) == -1) {
        	start = 0;
        	end = text.length();
        }
        
        while (text.indexOf("\n", start_index) != -1) {
        
        	if (count == index) {
        		
        		start = start_index;
        		
        		end_index = text.indexOf("\n", start_index+1);
        		if (end_index != -1) end = end_index;
        		else end = text.length();
        		break;
        	}
        	
        	start_index = text.indexOf("\n", start_index + 1);
        	count ++;
        }
        
        if (index > 0) start = start+1;
        
        line = text.substring(start, end);
        return line;
	}

	/**
	 * Marks all syllables of the specific line.
	 * @param index: index of the text line
	 */
	public void styleTextBackground(int index) {
		
		restStyleTextBackground();

        StyledDocument doc = ge.editor.getStyledDocument();
        int start = 0;
        int end = 0;
        int count = 0;
        int start_index = 0;
        int end_index = 0;
        String text = "";
        String[] words = null;
        String line = "";
        Style style = ge.editor.addStyle("MyHilite", null);
        
        int length = ge.editor.getDocument().getLength();
        try {
			text = ge.editor.getDocument().getText(0, length);
		} catch (BadLocationException e) {
			System.out.println(e);
		}
        

        //Only one line
        if (text.indexOf("\n", start_index) == -1) {
        	start = 0;
        	end = text.length();
        }
        
        while (text.indexOf("\n", start_index) != -1) {
        
        	if (count == index) {
        		
        		start = start_index;
        		
        		end_index = text.indexOf("\n", start_index+1);
        		if (end_index != -1) end = end_index;
        		else end = text.length();
        		break;
        	}
        	
        	start_index = text.indexOf("\n", start_index + 1);
        	count ++;
        }
        
        if (index > 0) start = start+1;
        
        line = text.substring(start, end);
        words = line.split(" ");
        boolean c = true;
        String hyph = "";
        String[] hyphList = null;
        int wordFullLength = 0;
        int start_hyph = 0;
        int counter = 0;
        String last_color = "";
        
        for (int i = 0; i < words.length; i++) {
            
            wordFullLength = words[i].length();

            String word = words[i].replaceAll("[^a-zA-Z]", "");
            if (wordlist.get(word) != null) {
            	hyph = wordlist.get(word);
            	hyphList = hyph.split("-");
            } else {
            	hyphList = word.split("  ");
            }
            
            start_hyph = start;
            
        	for (int x = 0; x < hyphList.length; x++) {
        		
        		if (ge.accentuationList.get(index) != null) {
        			
        			String metric = ge.accentuationList.get(index);
        			String a_letter = Character.toString(metric.charAt(counter));
        			
        			if (a_letter.equals("e")) {
        				
        				if (last_color.equals("green")) {
        					StyleConstants.setBackground(style, green_);
        					last_color = "green_";
        				} else if (last_color.equals("green_")) {
        					StyleConstants.setBackground(style, green);
        					last_color = "green";
        				} else {
        					StyleConstants.setBackground(style, green);
        					last_color = "green";
        				}
        			}
        			if (a_letter.equals("-")) {
        				if (last_color.equals("rot")) {
        					StyleConstants.setBackground(style, rot_);
        					last_color = "rot_";
        				} else if (last_color.equals("rot_")) {
        					StyleConstants.setBackground(style, rot);
        					last_color = "rot";
        				} else {
        					StyleConstants.setBackground(style, rot);
        					last_color = "rot";
        				}
        			}
        			
                } else {
	        		if (c) {
	        			StyleConstants.setBackground(style, rot);
	        			c = false;
	        		} else {
	        			StyleConstants.setBackground(style, green);
	        			c = true;
	        		}
                }

                doc.setCharacterAttributes(start_hyph, hyphList[x].length(), style, false);
                start_hyph = start_hyph + hyphList[x].length();
                
                counter ++;
        	}
        	
        	start = start + wordFullLength + 1;       		
        }
	}
	
	/**
	 * Resets all text markers.
	 */
	public void restStyleTextBackground() {
		
		StyledDocument doc = ge.editor.getStyledDocument();
        Style style = ge.editor.addStyle("MyHilite", null);
        StyleConstants.setBackground(style, Color.white);
        doc.setCharacterAttributes(0, ge.editor.getText().length(), style, false);
	}
}
